# Authors: Pablo Saavedra
# Contact: saavedra.pablo@gmail.com

"""
This package contains Pycomings DAO modules.
"""


class DAOEntry(object):

    """
    Abstract base class for pycomings DAOEntry.

    Each writer module or package must export a subclass also called
    'DAOEntry'.

    The `send()` method is the main entry point.
    """


    def create_table():
        """
        Normally not overridden or extended in subclasses.
        """
        raise NotImplementedError

    def create_entry(self, entry):
        """
        Normally not overridden or extended in subclasses.
        """
        raise NotImplementedError

    def get_entry(self, entry_id):
        """
        Normally not overridden or extended in subclasses.
        """
        raise NotImplementedError

    def get_entries(self, offset=None, max_items=None):
        """
        Normally not overridden or extended in subclasses.
        """
        raise NotImplementedError

    def update_entry(self, entry):
        """
        Normally not overridden or extended in subclasses.
        """
        raise NotImplementedError

    def delete_entry(self, entry_id):
        """
        Normally not overridden or extended in subclasses.
        """
        raise NotImplementedError

    def search_entry_by_path(self, path):
        """
        Normally not overridden or extended in subclasses.
        """
        raise NotImplementedError

    def search_entry_by_date(self, date):
        """
        Normally not overridden or extended in subclasses.
        """
        raise NotImplementedError





# Authors: Pablo Saavedra
# Contact: saavedra.pablo@gmail.com

"""
This package contains Pycomings Parsers modules.
"""

import time

class LogFileParser(object):

    """
    Base class for pycomings LogFileParsers.

    Each writer module or package must export a subclass also called
    'LogFileParsers'.

    The `send()` method is the main entry point.
    """

    mark_file=""
    jobs=[]
    stop=False

    def __init__(self, mark_file, jobs=[]):
        self.mark_file = mark_file
        self.jobs = jobs

    def parse_line(self, line):
        """
        Parse a log line.

        Normally overridden or extended in subclasses.
        """
        raise NotImplementedError

    def parse_lines(self, lines):
        """
        Parse a list of lines.

        Normally overridden or extended in subclasses.
        """
        raise NotImplementedError


    def save_mark(self,line):
      """ This function saves de last processed line """

      try:
        f = open(self.mark_file, 'w')
        f.write(line)
        f.close()
      except Exception,e:
        print e

    def load_mark(self):
      """ This function load de last processed line """

      try:
        f = open(self.mark_file)
        line = f.readline()
        f.close()
        return line
      except Exception,e:
        f = open(self.mark_file, 'w')
        f.close()
        print e
        return ""

      
    def watch_logs(self, logfile_names,sleeping_success,sleeping_no_incoming_lines):
        """
        While not "stop" value is active, then this method wacth the
        logfiles.
        """
        
        f,buffer_ = get_pending_lines(logfile_names)
        while not self.stop:
          if not len(buffer_) == 0:
           time.sleep(sleeping_no_incoming_lines)
           f,buffer_ = get_pending_lines(logfile_names)
          else:
           for l in buffer_:
               for j in self.jobs:
                   # Executed job for this line
                   j(l)
           self.save_mark(l)
           time.sleep(sleeping_success)
           buffer_ = f.readlines()


    def get_pending_lines(self, logfile_names):
        """
        Search pending lines to proccess. This function read lines for the 
        log file. then:

        - Search proccessed mark into the buffer:
 
          * if mark dont exist, return ''buffer''
          * if mark exist:

            * if mark is into main log file:
              buffer = buffer - previously processed lines.      
            * if it is founded into rotated log tipically (*.1):
              buffer = log rotated pending lines + buffer
            * if it is not founded into any logfile:
              buffer = log rotated lines + buffer
        
        Return: ( logfile[0] open, pending lines list)
        """

        f = open(logfile_names[0])
        buffer_ = f.readlines()

        mark =  self.load_mark()
        if not mark:
          print "Mark not founded"
          return (f,buffer_)

        # (else)
        try:
          # check if mark exist into buffer ...
          buffer_.index(mark)
          # ... this method can raise ValueError exception

          buffer_ = buffer_[buffer_.index(mark)+1:]
          # print "Mark founded into primarly log file"
          return (f,buffer_)

        except ValueError, e:

          # mark dont founded into buffer ...
          try:
            rotated_file=open(logfile_names[1])
            rotated_buffer=rotated_file.readlines()
  
            try:
              rotated_buffer.index(mark)
              buffer_ = \
                rotated_buffer[rotated_buffer.index(mark)+1:] \
                + buffer_
              # print "Mark founded into secondary log file"
              return (f,buffer_)
            except ValueError, e:
              # mark dont founded into rotated_buffer
              buffer_ = rotated_buffer + buffer_
              # print "Mark dont founded into primarly and secondary log file"
              return (f,buffer_)

          except Exception, e:
            # print "Can not open rotated log file."
            return (f,buffer_)

        raise Exception()



#!/usr/bin/env python
# vim:fenc=utf-8:tw=77:ts=2:
#
# by Pablo Abelenda (pablo.abelenda@lambdastream.com)
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
############################################################################# 


import unittest
import pyncomings
from pyncomings.dao import DAOsqlite3
from pyncomings.parsers import *
from xmldict.xmldict import *

import sqlite3

class testPyncomings(unittest.TestCase):

  """
  A test class for the Pyncomings software.
  """

  def setUp(self):
    """
    set up data used in the tests.
    setUp is called before each test function execution.
    """
    global e1
    global e2
    global search_result
    global query_result_e2_after_modification

    e1 = {'purged_date': 'e1', 'registration_date': 'e1', 'retention_days': 1, 'source': 'e1', 'user': 'e1', 'path': 'e1', 'size': 'e1'}
    e2 = {'purged_date': 'e2', 'registration_date': 'e2', 'retention_days': 2, 'source': 'e2', 'user': 'e2', 'path': 'e2', 'size': 'e2'}

    search_result = [{'purged_date': u'e1', 'registration_date': u'e1', 'retention_days': 1.0, 'source': u'e1', 'user': u'e1', 'path': u'e1', 'size': u'e1'}]
    query_result_e2_after_modification = {'purged_date': u'e2', 'registration_date': u'e2', 'retention_days': 2.0, 'source': u'e2', 'user': u'e2', 'path': u'e2', 'size': u'SIZE22'}

   
  def test_simple(self):
    """
    This test run, at least one time, all the functions defined in the module
    """
    global e1
    global e2
    global search_result
    global query_result_e2_after_modification

    d = DAOsqlite3.DAOEntry(":memory:")
    d.create_table()

    print "Getting a non existing entry, expected None as answer"
    self.assertEqual(d.get_entry(e1["path"]), None)

    print "Updating a non existing entry. Non error expected"
    d.update_entry(e1)
    
    print "Getting all entries. At this point, no one."
    self.assertEqual(d.get_entries(), [])

    print "Getting all entries passing an offset parameter. Exception expected."
    # self.assertRaises(d.get_entries(42))
    try:
        d.get_entries(42)
    except:
        NotImplementedError
    
    print "Creating a new entry -> e1"
    d.create_entry(e1)
    try:
      d.create_entry(e1)
      print "Error, can enter a entry with a path entry previously."
    except sqlite3.IntegrityError:
      print "OK: Integrity error success"

    print "Creating a new entry -> e2"
    d.create_entry(e2)

    print "Getting entries. At this point, e1 and e2, so, not None"
    self.assertNotEqual(d.get_entries(), None)

    print "Checking entry e1"
    self.assertEqual(d.get_entry(e1["path"]), e1)

    print "Checking entry e2"
    self.assertEqual(d.get_entry(e2["path"]), e2)

    print "Searching entries with 'registration_date' equal to e1 - only e1"
    self.assertEqual(d.search_entry_by_date(e1["registration_date"]),search_result)

    print "Copying e2 into e22 and changing e22 size"
    e22 = e2
    e22["size"]="SIZE22"
    d.update_entry(e22)

    print "Getting new entry e22"
    self.assertEqual(d.get_entry(e22["path"]), query_result_e2_after_modification)

    print "Deleting e1 entry"
    d.delete_entry(e1["path"])
    print "Getting e1 after deletion. None expected"
    self.assertEqual(d.get_entry(e1["path"]), None)


def suite():

    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(testPyncomings))
    return suite

if __name__ == '__main__':

    unittest.main()

    unittest.TextTestRunner(verbosity=2).run(suite())

  



# Authors: Pablo Saavedra
# Contact: saavedra.pablo@gmail.com

"""
This package contains wowza error log Parser module
 
Ref: http://wowza.com
"""


from pyncomings import parsers

import time

class LogFileParser(parsers.LogFileParser):
    """
>>> f = open("wowza_error_log")
>>> l = f.readline()
>>> l
'WARN\tserver\tcomment\t2014-12-12\t06:01:50\t-\t-\t-\t-\t-\t40.119\t-\t-\t-\t-\t-\t-\t-\t-\tDvrStreamStoreBase.storeChunks[str001/_definst_/MBR_TNT_SD_Audio1_800000.stream/MBR_TNT_SD_Audio1_800000.stream.0] : Skipping chunk. A/V packet times differ by 12774 ms, more than allowed 10000 ms.   aTime=82213197 vTime=82225971\n'
>>> l.split("\t")
['WARN', 'server', 'comment', '2014-12-12', '06:01:50', '-', '-', '-', '-', '-', '40.119', '-', '-', '-', '-', '-', '-', '-', '-', 'DvrStreamStoreBase.storeChunks[str001/_definst_/MBR_TNT_SD_Audio1_800000.stream/MBR_TNT_SD_Audio1_800000.stream.0] : Skipping chunk. A/V packet times differ by 12774 ms, more than allowed 10000 ms.   aTime=82213197 vTime=82225971\n']
>>> l.split("\t")[0]
'WARN'
>>> l.split("\t")[1]
'server'
>>> l.split("\t")[2]
'comment'
>>> l.split("\t")[3]
'2014-12-12'
>>> l.split("\t")[4]
'06:01:50'
>>> l.split("\t")[19]
'DvrStreamStoreBase.storeChunks[str001/_definst_/MBR_TNT_SD_Audio1_800000.stream/MBR_TNT_SD_Audio1_800000.stream.0] : Skipping chunk. A/V packet times differ by 12774 ms, more than allowed 10000 ms.   aTime=82213197 vTime=82225971\n'
    """

    def __init__(self, mark_file, jobs=[]):
        super(LogFileParser, self).__init__(mark_file, jobs)


    def parse_line(self, line):
        """
        Parse a log line.
        """

        line_items = line.split("\t")

        try:
            entry = {
                  "entry_id": None,
                  "registration_date": time.time() ,
                  "source": line_items[1],
                  "size": "",
                  "path": line_items[19],
                  "user": line_items[2],
                  "retention_days": 0,
                  "purged_date": False
                }
        except Exception:
            entry = {
                  "entry_id": None,
                  "registration_date": time.time() ,
                  "source": "",
                  "size": "",
                  "path": "",
                  "user": "",
                  "retention_days": 0,
                  "purged_date": False
            }

        return entry



# Authors: Pablo Saavedra
# Contact: saavedra.pablo@gmail.com

"""
This package contains XFERLOG Parser module
 
Ref: http://compute.cnr.berkeley.edu/cgi-bin/man-cgi?xferlog+4 
"""


from pyncomings import parsers

import time

class LogFileParser(parsers.LogFileParser):

        
    def __init__(self, mark_file, jobs=[]):
        super(LogFileParser, self).__init__(mark_file, jobs)

 
    def parse_line(self, line):
        """
        Parse a log line.
        """
       
        line_items = line.split()

        entry = {
                  "entry_id": None,
                  "registration_date": time.time() ,
                  "source": line_items[6],
                  "size": line_items[7],
                  "path": line_items[8],
                  "user": line_items[13],
                  "retention_days": 0,
                  "purged_date": False
                }
        return entry
